var knownSettings = {};
var lastCurrentValue = null;

var main = document.createElement("main");
document.body.appendChild(main);

var unorderedlist = document.createElement("ul");
main.appendChild(unorderedlist);

var nextStyle = document.getElementById("nextStyle");

setTimeout(
  () =>
    sendThroughWebSocket(
      JSON.stringify({
        type: "SettingsRequest",
      })
    ),
  50
);

function handleMessage(message) {
  if (message.type == "MessageBroadcast")
    try {
      handleBroadcastedMessage(JSON.parse(message.message));
    } catch (e) {
      console.error(message);
      console.error(e);
    }
  if (message.type == "YouAre")
    setTimeout(
      () =>
        sendThroughWebSocket(
          JSON.stringify({
            type: "SettingsRequest",
          })
        ),
      10
    );
  if (message.type == "ClientConnected") {
    if (!knownSettings)
      setTimeout(
        () =>
          sendThroughWebSocket(
            JSON.stringify({
              type: "SettingsRequest",
            })
          ),
        10
      );
  }
}

function handleBroadcastedMessage(message) {
  if (message.type == "Settings" || message.type == "NewSettings") {
    Object.entries(message.settings).forEach(
      ([k, v]) => (knownSettings[k] = v)
    );
    configureForSettings();
  } else if (message.type == "SettingsRequest") {
    if (knownSettings)
      sendThroughWebSocket(
        JSON.stringify({
          type: "Settings",
          settings: knownSettings,
        })
      );
  }
  console.log(message);
}

var escapeHtml = (unsafe) =>
  unsafe
    .replace(/&/g, "&amp;")
    .replace(/</g, "&lt;")
    .replace(/>/g, "&gt;")
    .replace(/"/g, "&quot;")
    .replace(/'/g, "&#039;");

class AsyncLazyEval {
  /**
   * @param {Promise<Object>} promise
   */
  constructor(promise) {
    this._promise = promise;
    this._evaluating = false;
    this._evaluated = false;
    this._thrown = false;
    this._value = null;
  }

  get eval() {
    if (this._evaluated)
      return new Promise((resolve, reject) => {
        if (!this._thrown) resolve(this._value);
        else reject(this._value);
      });
    else if (!this._evaluating) {
      this._evaluating = true;
      return new Promise((resolve, reject) => {
        this._promise
          .then((r) => {
            this._evaluating = false;
            this._evaluated = true;
            this._thrown = false;
            this._value = r;
            resolve(r);
          })
          .catch((r) => {
            this._evaluating = false;
            this._evaluated = true;
            this._thrown = true;
            this._value = r;
            reject(r);
          });
      });
    } else {
      let p = new Promise(async (resolve, reject) => {
        while (!this._evaluated) await asyncSleep(100);
        if (!this._thrown) resolve(this._value);
        else reject(this._value);
      });
      return p;
    }
  }
}

var conventionJsonCache = {};

/**
 * @param {number} ms
 * @returns {Promise<void>}
 */
function asyncSleep(ms) {
  return new Promise((resolve) => setTimeout(resolve, ms));
}

setTimeout(async () => {
  while (1) {
    try {
      await configureForSettings();
    } catch (e) {}
    await asyncSleep(20 * 1000);
  }
}, 50);

/**
 * @param {string} url
 */
async function ensureCacheUpdated(url) {
  var cached = conventionJsonCache[knownSettings.scheduleUrl];
  var time = new Date().getTime();
  if (!cached || (!!cached && time > cached.taken + 5 * 60 * 1000))
    cached = conventionJsonCache[knownSettings.scheduleUrl] = {
      taken: new Date().getTime(),
      data: new AsyncLazyEval(
        (async () => {
          if (url === undefined) throw new Error("Url cannot be undefined");
          const response = await fetch(
            `${url}${url?.includes("?") ? "&" : "?"}nonce=${time}`
          );
          const jsonData = await response.json();
          return jsonData;
        })()
      ),
    };
  try {
    return await cached.data.eval;
  } catch (e) {
    console.error(e);
    return undefined;
  }
}

/**
 * @param {number} n
 * @returns {number[]}
 */
function countUntil(n) {
  return Array(n)
    .fill(0)
    .map((_, i) => i);
}

function lpad(text, size, fill) {
  var t = String(text);
  while (t.length < size) t = `${fill}${t}`;
  return t;
}

var happeningNow = undefined;
var happeningSoon = undefined;
var selectedHappening = undefined;

function rangeOverlap(this_start, this_finish, that_start, that_finish) {
  return (
    (this_start >= that_start && this_start < that_finish) ||
    (this_finish > that_start && this_finish < that_finish) ||
    (that_start >= this_start && that_start < this_finish) ||
    (that_finish > this_start && that_finish < this_finish)
  );
}

async function configureForSettings() {
  if (!!knownSettings) {
    var cached = await ensureCacheUpdated(knownSettings.scheduleUrl);
    var nowMillis =
      knownSettings.dateOverrideBool === "Y"
        ? new Date(knownSettings.dateOverride).getTime()
        : new Date().getTime();
    if (!!cached) {
      console.log(knownSettings.scheduleUrl, cached);
      var lang = knownSettings.language || cached.language;
      var conventionEdition = cached.editions.filter(
        (x) => x.uuid === cached.featured
      )[0];
      var conventionPlaces = Object.fromEntries(
        conventionEdition.places.map((x) => [x.uuid, x.label[lang]])
      );
      var splitDayIntoNParts = conventionEdition.split_day_into_n_parts;
      var todayStart =
        Math.floor(nowMillis / (24 * 3600 * 1000)) * (24 * 3600 * 1000);
      var dayPartDuration = (24 * 3600 * 1000) / splitDayIntoNParts;
      var dayParts = countUntil(splitDayIntoNParts).map(
        (x) => todayStart + x * dayPartDuration
      );
      var thisDayPart = dayParts.filter(
        (x) => x <= nowMillis && nowMillis < x + dayPartDuration
      )[0];
      var nextDayPart = thisDayPart + dayPartDuration;
      var nextDayPartEnd = nextDayPart + dayPartDuration;
      var events = conventionEdition.events.map((x) => ({
        millis_start: new Date(
          x.time_start.yr,
          x.time_start.mo - 1,
          x.time_start.dy,
          x.time_start.hr,
          x.time_start.mn,
          x.time_start.sc
        ).getTime(),
        millis_end: new Date(
          x.time_end.yr,
          x.time_end.mo - 1,
          x.time_end.dy,
          x.time_end.hr,
          x.time_end.mn,
          x.time_end.sc
        ).getTime(),
        summary:
          x?.subtitle?.[lang] ||
          x?.description?.[lang] ||
          knownSettings.fallbackDescription ||
          "???",
        placeNames: x.places.map((x) => conventionPlaces[x]),
        titleLang: x.title[lang],
        ...x,
      }));
      happeningNow = events.filter((x) =>
        rangeOverlap(x.millis_start, x.millis_end, thisDayPart, nextDayPart)
      );
      happeningSoon = events.filter((x) =>
        rangeOverlap(x.millis_start, x.millis_end, nextDayPart, nextDayPartEnd)
      );
      selectedHappening = (
        knownSettings.tab === "current" ? happeningNow : happeningSoon
      ).sort((a, b) =>
        a.millis_start != b.millis_start
          ? a.millis_start - b.millis_start
          : a.millis_end != b.millis_end
          ? a.millis_end - b.millis_end
          : a.titleLang > b.titleLang
          ? 1
          : -1
      );
      console.log(selectedHappening);
    }
    while (unorderedlist.lastChild)
      unorderedlist.removeChild(unorderedlist.lastChild);
    for (var conventionEvent of selectedHappening) {
      const listItem = document.createElement("li");
      const timeContainer = document.createElement("div");
      const timeStartContainer = document.createElement("div");
      const timeFinishContainer = document.createElement("div");
      timeContainer.classList.add("time");
      timeStartContainer.classList.add("timeStart");
      timeFinishContainer.classList.add("timeFinish");
      timeContainer.appendChild(timeStartContainer);
      timeContainer.appendChild(timeFinishContainer);
      listItem.appendChild(timeContainer);
      const detailsContainer = document.createElement("div");
      const line1Container = document.createElement("div");
      const line2Container = document.createElement("div");
      const line3Container = document.createElement("div");
      detailsContainer.classList.add("details");
      line1Container.classList.add("title");
      line2Container.classList.add("subtitle");
      line3Container.classList.add("place");
      detailsContainer.appendChild(line1Container);
      detailsContainer.appendChild(line2Container);
      detailsContainer.appendChild(line3Container);
      listItem.appendChild(detailsContainer);
      unorderedlist.appendChild(listItem);
      timeStartContainer.innerText = `${lpad(
        conventionEvent.time_start.hr,
        2,
        0
      )}:${lpad(conventionEvent.time_start.mn, 2, 0)}`;
      timeFinishContainer.innerText = `${lpad(
        conventionEvent.time_end.hr,
        2,
        0
      )}:${lpad(conventionEvent.time_end.mn, 2, 0)}`;
      line1Container.innerText = String(conventionEvent.titleLang);
      line2Container.innerText = String(
        conventionEvent.summary.length > 50
          ? conventionEvent.summary.substring(0, 47) + "..."
          : conventionEvent.summary
      );
      line3Container.innerText = String(conventionEvent.placeNames.join(", "));
    }
    nextStyle.innerHTML = `
      html, body {
        margin: 0;
        padding: 0;
      }
      body > main > ul {
        margin: 0;
        padding: 0;
        list-style-type: none;
        font-family: ${knownSettings.fontFamily ?? "sans-serif"};
        font-weight: ${knownSettings.baseFontWeight ?? 500};
        font-size: ${knownSettings.baseFontSize ?? 1.2}rem;
      }
      body > main > ul > li {
        margin: ${knownSettings.cardMargin ?? 1}em;
        padding: ${knownSettings.cardPadding ?? 0.5}em;
        border-radius: ${knownSettings.cardBorderRadius ?? 0.25}em;
        background: ${knownSettings.cardColor ?? "#00000080"};
        overflow: hidden;
        display: flex;
        flex-direction: row;
        align-content: stretch;
        justify-content: stretch;
      }
      body > main > ul > li > div.time {
        margin-right: ${knownSettings.cardPadding ?? 0.5}em;
        display: flex;
        width: ${knownSettings.timeWidth ?? 4}em;
        flex-direction: column;
        justify-content: center;
        text-align: center;
      }
      body > main > ul > li > div.time > div.timeStart {
        color: ${knownSettings.timeStartColor ?? "#1a1a1a"};
      }
      body > main > ul > li > div.time > div.timeFinish {
        color: ${knownSettings.timeFinishColor ?? "#333333"};
      }
      body > main > ul > li > div.details {
        white-space: nowrap;
        text-overflow: ellipsis;
      }
      body > main > ul > li > div.details > div.title {
        font-size: ${knownSettings.titleFontSize ?? 1.5}em;
        color: ${knownSettings.titleFontColor ?? "#000000"};
      }
      body > main > ul > li > div.details > div.subtitle {
        font-style: italic;
        font-size: ${knownSettings.subtitleFontSize ?? 0.85}em;
        color: ${knownSettings.subtitleFontColor ?? "#333333"};
      }
      body > main > ul > li > div.details > div.place {
        font-size: ${knownSettings.placeFontSize ?? 0.75}em;
        color: ${knownSettings.placeFontColor ?? "#1a1a1a"};
      }
    `.trim();
  }
}

var incrementDisplayBuffer = [];
function displayIncrement(formatted) {
  if (!!formatted) {
    var emptyBuffer = incrementDisplayBuffer.length == 0;
    incrementDisplayBuffer.push(formatted);
    if (emptyBuffer) processIncrementDisplayBuffer();
  }
}

function processIncrementDisplayBuffer() {
  if (incrementDisplayBuffer.length > 0) {
    var toDisplay = incrementDisplayBuffer[0];
    increment.innerText = toDisplay;
    increment.classList.remove("transparent");
    setTimeout(() => {
      setTimeout(() => {
        increment.classList.add("transparent");
        setTimeout(() => {
          increment.innerText = "";
          incrementDisplayBuffer.shift();
          processIncrementDisplayBuffer();
        }, knownSettings.transitionsDuration);
      }, knownSettings.diffDuration * 1000);
    }, knownSettings.transitionsDuration);
  }
}
